"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Connect = void 0;
const react_1 = require("react");
const connect_1 = require("@1hive/connect");
const connect_context_1 = require("./connect-context");
const utils_1 = require("./utils");
function orgStatusReducer(state, action) {
    if (action.type === 'loading') {
        return state.orgLoading
            ? state
            : { org: null, orgError: null, orgLoading: true };
    }
    if (action.type === 'error') {
        return state.orgError === action.error
            ? state
            : { org: null, orgError: action.error, orgLoading: false };
    }
    if (action.type === 'done') {
        return state.org === action.org
            ? state
            : { org: action.org, orgError: null, orgLoading: false };
    }
    return state;
}
const orgStatusInitial = {
    org: null,
    orgError: null,
    orgLoading: true,
};
function Connect({ children, connector, location, options, }) {
    const [{ org, orgError, orgLoading }, setOrgStatus] = react_1.useReducer(orgStatusReducer, orgStatusInitial);
    const cancelOrgLoading = react_1.useRef(null);
    const connectorUpdateValue = Array.isArray(connector) || typeof connector === 'string'
        ? utils_1.hash(connector)
        : // If the connector is custom instance,
            // it is the responsibility of the provider to memoize it.
            connector;
    const loadOrg = react_1.useCallback(() => {
        var _a;
        let cancelled = false;
        (_a = cancelOrgLoading.current) === null || _a === void 0 ? void 0 : _a.call(cancelOrgLoading);
        cancelOrgLoading.current = () => {
            cancelled = true;
        };
        setOrgStatus({ type: 'loading' });
        const update = async () => {
            try {
                const org = await connect_1.connect(location, connector, options);
                if (!cancelled) {
                    setOrgStatus({ type: 'done', org });
                }
            }
            catch (err) {
                if (!cancelled) {
                    setOrgStatus({ type: 'error', error: err });
                }
            }
        };
        update();
    }, [location, connectorUpdateValue, utils_1.hash(options)]);
    react_1.useEffect(loadOrg, [loadOrg]);
    const value = react_1.useMemo(() => ({
        org,
        orgStatus: {
            error: orgError,
            loading: orgLoading,
            retry: loadOrg,
        },
    }), [org, orgError, orgLoading, loadOrg]);
    return react_1.createElement(connect_context_1.ConnectContext.Provider, { value, children });
}
exports.Connect = Connect;
//# sourceMappingURL=Connect.js.map