"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useConnect = void 0;
const react_1 = require("react");
const types_1 = require("./types");
const connect_context_1 = require("./connect-context");
const utils_1 = require("./utils");
function statusReducer(state, action) {
    if (action.type === 'loading') {
        return state.loading
            ? state
            : { error: null, loading: true, result: undefined };
    }
    if (action.type === 'error') {
        return state.error === action.error
            ? state
            : { error: action.error, loading: false, result: undefined };
    }
    if (action.type === 'done') {
        return utils_1.hash(state.result) === utils_1.hash(action.result)
            ? state
            : { error: null, loading: false, result: action.result };
    }
    return state;
}
const statusInitial = {
    error: null,
    loading: true,
    result: undefined,
};
function useConnect(callback, dependencies) {
    const { org, orgStatus } = connect_context_1.useConnectContext();
    const [{ result, error, loading }, setStatus] = react_1.useReducer(statusReducer, statusInitial);
    if (callback === undefined) {
        callback = ((org) => org);
    }
    react_1.useEffect(() => {
        let cancelled = false;
        let subscriptionHandler = null;
        const update = async () => {
            setStatus({ type: 'loading' });
            try {
                if (!org) {
                    if (orgStatus.error !== null) {
                        setStatus({ type: 'error', error: orgStatus.error });
                    }
                    return;
                }
                const result = await callback(org);
                if (cancelled || result === undefined) {
                    return;
                }
                // Subscription
                if (types_1.isSubscriptionStart(result)) {
                    subscriptionHandler = result((error, result) => {
                        if (cancelled) {
                            return;
                        }
                        if (error) {
                            setStatus({ type: 'error', error });
                            return;
                        }
                        setStatus({ type: 'done', result });
                    });
                    // Just async data
                }
                else {
                    setStatus({ type: 'done', result });
                }
            }
            catch (err) {
                if (!cancelled) {
                    setStatus({ type: 'error', error: err });
                }
            }
        };
        update();
        return () => {
            cancelled = true;
            subscriptionHandler === null || subscriptionHandler === void 0 ? void 0 : subscriptionHandler.unsubscribe();
        };
    }, [org, ...(dependencies || [])]);
    return react_1.useMemo(() => [result, { error, loading, retry: () => null }], [result, error, loading]);
}
exports.useConnect = useConnect;
//# sourceMappingURL=use-connect.js.map