import { useEffect, useRef, useState } from 'react';
import { warn } from '@1hive/connect-core';
import { useConnect } from './use-connect';
import { hash } from './utils';
export function createAppHook(appConnect, connector) {
    warn('createAppHook is going to be deprecated in the next version. ' +
        'Please use useConnect() and the appConnectors prop on <Connect />.');
    return function useAppData(app, callback, dependencies) {
        // Store the callback as a ref so consumers can declare it inline.
        const callbackRef = useRef((app) => app);
        useEffect(() => {
            callbackRef.current = callback || ((app) => app);
        }, [callback]);
        const [[connectedApp, appError], setApp] = useState([null, null]);
        const appHashRef = useRef('');
        useEffect(() => {
            const appHash = hash(app);
            // Only update the app when it has changed.
            if (appHash === appHashRef.current) {
                return;
            }
            appHashRef.current = appHash;
            // Don’t try to connect if the app doesn’t exist yet
            if (!app) {
                return;
            }
            let cancelled = false;
            appConnect(app, connector)
                .then((connectedApp) => {
                if (!cancelled) {
                    setApp([connectedApp, null]);
                }
            })
                .catch((err) => {
                if (!cancelled) {
                    setApp([null, err]);
                }
            });
            return () => {
                cancelled = true;
            };
        }, [app]);
        return useConnect(() => {
            if (appError) {
                throw appError;
            }
            return connectedApp ? callbackRef.current(connectedApp) : undefined;
        }, [...(dependencies || []), connectedApp, appError]);
    };
}
//# sourceMappingURL=create-app-hook.js.map